package logic;

import java.util.*;

public class Algorithm_fewStars extends Algorithm {
    public static final String NAME = "logic.Algorithm_fewStars";
    public static final int MAX_NUMBER_CONDITIONS = 31;    
    private Action[] ruleTable;
    public Algorithm_fewStars() {
        super();
    }
    
    public void init(int numConditions, int numRules, int maxNumStars, boolean consisCheck) {
        super.init(numConditions,numRules,maxNumStars,consisCheck);
        
        Util.truthTable(maxNumStars);
        ruleTable = new Action[(int)Math.pow(2,numConditions)];
        
    }
    
    /** 
     *  This method adds a new boolean-sequence / action pair
     *  to the rules Hashtable. It first converts the String boolStr,
     *  which is a string of T's and F's, to a BooleanTuple, then
     *  it loads this BooleanTuple as a key in ruleTable, matched
     *  with the action argument passed in.
     */
    public void addRule(String boolStr, Action action) 
        throws PoorPerformanceException, InconsistentRulesException {
        
        BooleanTupleFactory.loadTuples(boolStr,this,action);
    }
    
    
    
    /**
     *  This method looks up, in ruleTable, 
     *  the action that corresponds to the BooleanTuple
     *  passed in.
     */
    
    public Action lookUp(Vector results)throws RuleNotFoundException,PoorPerformanceException, InconsistentRulesException,RuleNotFoundException {
        BooleanTuple bt = new BooleanTuple(results);
        int hash = bt.hashCode();
        try {
            Action action = null;
            if( (action = ruleTable[hash]) == null) {
                throw new RuleNotFoundException();
            }
            return action;
        }
        catch(ArrayIndexOutOfBoundsException e) {
            throw new RuleNotFoundException();
        }
        
        
    }
    
    public String name() {
        return NAME;
    }
    
    public Action [] getRuleTable() {
        return ruleTable;
    }
    public void setRuleTable(Action [] rt) {
        ruleTable = rt;
    }
   
}