package logic;

import java.util.*;

public class Algorithm_manyStars extends Algorithm {
    public static final String NAME = "logic.Algorithm_manyStars"; 
    public static final int MAX_NUMBER_CONDITIONS = 15000;    
    
    /** initialize table so that it can handle a fairly large number of rules--need to make sure
     *  that its size is a prime (and 1999 is a prime)
     */
	private Hashtable ruleTable = new Hashtable(1999);
	private String[] boolStrings;
	private int nextAvailableIndex = 0;
    
    public Algorithm_manyStars() {
        super();
        
    }
    public void init(int numConditions, int numRules,int maxNumStars, boolean consisCheck){
        super.init(numConditions, numRules, maxNumStars,consisCheck);
        boolStrings = new String[numRules];
        
    }
    
    public String name() {
        return NAME;
    }    
    
	/** 
	 *  This method adds a new boolean-sequence / action pair
	 *  to the rules Hashtable. It first converts the String boolStr,
	 *  which is a string of T's and F's, to a BooleanTuple, then
	 *  it loads this BooleanTuple as a key in ruleTable, matched
	 *  with the action argument passed in.
	 */
	public void addRule(String boolStr, Action action) throws InconsistentRulesException, 
	                                                          PoorPerformanceException,
	                                                          RuleNotFoundException {
	    if(boolStr.length() > MAX_NUMBER_CONDITIONS) 
	        throw new PoorPerformanceException();
	    if(consistencyChecking) {
	        for(int i = 0; i < nextAvailableIndex; ++i) {
	            if(Util.equals(boolStrings[i], boolStr) &&
	               !(action.equals((Action)ruleTable.get(boolStrings[i])))) {
	                    throw new InconsistentRulesException("Boolean string "+boolStr+" is being "+
	                                                         "associated with different actions.");
	            }
	        }
	    }
	        
	    boolStrings[nextAvailableIndex++] = boolStr;
	    ruleTable.put(boolStr,action);
	}
	/**
	 *  This method looks up, in ruleTable, 
	 *  the action that corresponds to the BooleanTuple
	 *  passed in.
	 */
	public Action lookUp(Vector results) throws RuleNotFoundException,PoorPerformanceException, InconsistentRulesException,RuleNotFoundException {
	    
	    //convert results vector to a boolean string
	    String boolStr = Util.booleansToString(results);
	    
	    //first try to find a match between boolString and one
	    //of the starred boolean strings
	    String starredMatch = null;
	    for(int i = 0; i < boolStrings.length; ++i) {
	        if(Util.equals(boolStr, boolStrings[i])) {
	            starredMatch = boolStrings[i];
	            break;
	        }
	    }
	    if(starredMatch == null) {
	        throw new RuleNotFoundException("In Rules, can't find a match for "+boolStr);
	    }
	    
	    //now look up action corresponding to starredMatch
	    Object ob = ruleTable.get(starredMatch);
	    if(ob == null) {
	        throw new RuleNotFoundException(starredMatch);
	    }
	    return (Action)ob;
	}    
}