    package logic;
    import java.util.*;

    /**
     *  This class encapsulates the notion of a set of ConditionValues
     *  Provides add, remove methods; has a contains() query method; and implements
     *  Enumeration for iteration. Note that one feature of a set is that it doesn't allow
     *  duplicate elements. 
     *  <p>
     *  @author Paul Corazza
     */
    public class ConditionValueSet implements Enumeration {
        private Vector setMembers;
        private int iterator;
    
        public ConditionValueSet() {
            setMembers = new Vector();
        }
        public ConditionValueSet(ConditionValue cv) {
            this();
            addMember(cv);
        }
        public boolean addMember(ConditionValue cv) {
            if (!contains(cv)) {
                setMembers.addElement(cv);
                return true;
            }
            return false;
        }
        public boolean contains(ConditionValue cv) {
            if (setMembers == null || setMembers.isEmpty()) {
                return false;
            }
            int size = setMembers.size();
            ConditionValue aCV = null;
            for (int i = 0; i < size; ++i) {
                aCV = (ConditionValue) setMembers.elementAt(i);
                if(cv.equals(aCV)) return true;
            }
            return false;
        }
        public int getCardinality() {
            return setMembers.size();
        }
        public ConditionValue[] getElements() {
            if (setMembers == null)
                return null;
            int size = setMembers.size();
            ConditionValue[] retVal = new ConditionValue[size];
            ConditionValue aCV = null;
            for (int i = 0; i < size; ++i) {
                aCV = (ConditionValue) setMembers.elementAt(i);
                retVal[i] = aCV;
            }
            return retVal;
        }
        /** you can call hasMoreElements() repeatedly and get same result; 
        *  iterator not incremented
        *  by simply reading
        */
        public boolean hasMoreElements() {
            return (iterator < setMembers.size());
        }
        private void incrementIterator() {
            ++iterator;
        }
        /**
        *  returns the element at current index and then increments iterator
        *  returns null if you try to use an index that's not allowed
        */
        public Object nextElement() {
            try {
                Object ob = setMembers.elementAt(iterator);
                incrementIterator();
                return ob;
            } 
            catch (Exception e) {
                return null;
            }
        }
        public boolean removeElement(ConditionValue cv) {
            if (setMembers == null){
                return false;
            }
            int size = setMembers.size();
            ConditionValue current = null;
            for (int i = 0; i < size; ++i) {
                current = (ConditionValue) setMembers.elementAt(i);
                if (current.equals(cv)) {
                    setMembers.removeElementAt(i);
                    return true;
                }
            }
            return false;
        }
        //iterator methods

        public void resetIterator() {
            iterator = 0;
        }
        public String toString() {
            String newline = System.getProperty("line.separator");
            StringBuffer sb = new StringBuffer("{"+newline);
            int count = 0;
            for (resetIterator(); hasMoreElements();) {
                sb.append("  ");
                sb.append((ConditionValue)nextElement());
                sb.append(",");
                sb.append(newline);
            }
            
            String retVal = sb.toString();
            retVal = retVal.substring(0, retVal.length() - 3);
            retVal += (newline + "}");
            return retVal;
        }
        /*
        public static void main(String[] args) {
            testToString();
        }
        
        public static void testToString() {
            try {
                Condition c1 = new Condition(1,Condition.LESS, 3);
                Condition c2 = new Condition(2.0f, Condition.LESS_EQUAL, 1.0f);
                Condition c3 = new Condition(3.5, Condition.EQUAL, 3.50);
                Condition c4 = new Condition(true, false);
                Condition c5 = new Condition("hello","hello");
                Hashtable aHT = new Hashtable(5);
                aHT.put("one","One");
                aHT.put("two","Two");
                Condition c6= new Condition(aHT);
            
                ConditionValue cv1 = new ConditionValue(c1,true);
                ConditionValue cv2 = new ConditionValue(c2,false);
                ConditionValue cv3 = new ConditionValue(c3,true);
                ConditionValue cv4 = new ConditionValue(c4,false);
                ConditionValue cv5 = new ConditionValue(c5,true);
                ConditionValue cv6 = new ConditionValue(c6,true);
            
                ConditionValueSet set = new ConditionValueSet(cv1);
                set.addMember(cv2);
                set.addMember(cv3);
                set.addMember(cv4);
                set.addMember(cv6);
                set.addMember(cv5);
            
                System.out.println(set);
                System.in.read();
            }
            catch(Exception e){}
                        
        }*/
}
