package logic;

import java.util.*;

/**
 *  This is the "rules engine". It evaluates the sequence of conditions
 *  that is passed in, looks up the corresponding action in its rules
 *  table, and sends a message to that action to execute itself
 */
public class IfThenElse {

	
	/** The Vector of conditions represent a single specific sequence of conditions we
	 *  are interested in evaluating and performing an action on
	 *  The rules are a map that relates each sequence of Booleans
	 *  to an action that should be fired.
	 */
	public IfThenElse(Vector conditions, Rules rules) 
	    throws DataNotFoundException, PoorPerformanceException, RuleNotFoundException, InconsistentRulesException {
	        
		    Vector results = evaluateConditions(conditions);
		    executeAction(results, rules);
	}
	
	/**
	 *  This method evaluates each Condition in the Vector argument and
	 *  loads the resulting Booleans in a Vector that is ultimately returned.
	 */
	public Vector evaluateConditions(Vector conditions) throws DataNotFoundException {
		Vector results = new Vector(conditions.size());
		for(int i = 0; i < conditions.size(); ++i) {
			Condition c = (Condition)conditions.elementAt(i);
			Boolean result = c.evaluate();
			results.addElement(result);
		}
		return results;
	}
	
	/**
	 *  This method takes the Vector of results obtained from evaluateConditions(..)
	 *  and wraps it in a BooleanTuple. The BooleanTuple is looked up on
	 *  the rules table to obtain the corresponding Action instance.
	 *  This action instance is then requested to execute.
	 */
	public void executeAction(Vector results, Rules rules) 
	    throws PoorPerformanceException, RuleNotFoundException, InconsistentRulesException {
	        
		    Action action = rules.lookUp(results);
		    action.execute();
	}
	
}