package logic;

import java.util.*;

/**
 *  Some utilities
 */
public class Util {
    public static int[][][] TRUTH_TABLES;
    
    /**
     *  Returns 'T' if boolean argument is true, 'F' otherwise
     */
    public static char charVal(boolean b) {
        return b ? 'T' : 'F';
    }
    /**
     *  Returns true if x < y < z, false otherwise
     */
    public static boolean isBetween(int x, int y, int z) {
        return (x < y && y < z);
    }
    public static boolean isBetweenOrEqualOnRight(int x, int y, int z) {
        return (isBetween(x,y,z) || (y == z));
    }
    
    /** this method returns largest int not greater than log_2(num),
     *  for num >= 1, 0 otherwise
     */
    public static int logBase2AsInt(double num) {
        if(num <= 1) return 0;
        int k = 0;
        while(true) {
            if(Math.pow(2,k+1) > num) {
                return k;
            }
            k++;
        }
        
    }
        
    /**
     *  Arguments to this method are Strings composed of the characters
     *  'T', 'F' or '*'. Two such Strings of equal length will be considered unequal
     *  just in case for some i, the ith character of one String is a 'T'
     *  and the ith character of the other String is an 'F'
     */
	public static boolean equals(String boolStrA, String boolStrB) {
	    String a = boolStrA;
	    String b = boolStrB;
	    if(a == null || b == null || a.length() != b.length()) {
	        return false;
	    }
	    for(int i = 0; i < a.length(); ++i) {
	        //either T or F matches with a '*' -- these are the only ways matching fails
	        if( a.charAt(i) == 'T'  &&  b.charAt(i) == 'F' ||
	            a.charAt(i) == 'F'  &&  b.charAt(i) == 'T') {
	                
	                return false;
	        }
	    }
	    return true;
	}
    
    
	/**
	 *  Returns -1 if val is not one of the elements of intArr; otherwise
	 *  returns the index of val in intArr.
	 */
	public static int findIndex(int val, int[] intArr){
	    if(intArr == null) return -1;
	    for(int i = 0; i < intArr.length; ++i) {
	        if(val == intArr[i]) return i;
	    }
	    return -1;
	}
	
	public static String booleansToString(Vector results) {
	    if(results == null) return null;
	    int size = results.size();
	    StringBuffer sb = new StringBuffer(size);
	    for(int i = 0; i < size; ++i){
	        Boolean b = (Boolean)results.elementAt(i);
	        sb.append(charVal(b.booleanValue()));
	    }
	    return sb.toString();
	}
	        
	
    /**
    * Returns an array of Booleans having length the parameter passed in; 
    * if any element of Vector parameter
    * is not a Boolean, null is inserted into the array.
    * @author Paul Corazza
    * @return java.lang.Boolean[]
    * @param v java.util.Vector
    */
    public static Boolean [] toBooleanArray(Vector v) {
	    if(v == null) return null;
	    Boolean [] retVal = new Boolean[v.size()];
	    Object iter = null;
	    for(int i = 0; i < v.size(); ++i) {
    		iter = v.elementAt(i);
		    retVal[i] = (iter instanceof Boolean) ? (Boolean)iter : null;
	    }
    	
    	return retVal;
    }
    /** This method creates truth tables for i propositional symbols,
     *  where i ranges from 1 through numSymbols, and serves as the
     *  first index in a 3-d array. The middle index refers to the column
     *  of the ith table, and the last index refers to the row.
     *  Truth values as stored as 0's (false) and 1's (true).
     *  The 3-d array is stored in the static variable truthTables.
     */
    public static synchronized void truthTable(int numSymbols) {
        //if we already have a truth table with numSymbols many propositional
        //symbols, then return
        if(TRUTH_TABLES != null && TRUTH_TABLES.length >= (numSymbols+1)) {
            return;
        }
        int retVal[][][] = new int[numSymbols+1][][];
        int numRows = 1;
        int numRowsOld = 0;
        int numCols = 0;
        //no truth table for i = 0 -- "no propositional symbols"
        for(int i = 1; i < retVal.length; ++i) {
            numCols = i;
            numRowsOld = numRows;
            numRows = numRows + numRows;
            
            retVal[i] = new int[numCols][numRows];
            int [] powers = new int[numCols];
            powers[0] = 1;
            for(int j = 0; j < numCols; ++j) {
                if(j>0) powers[j] = powers[j-1] + powers[j-1];
                for(int k = 0; k < numRows; ++k) {
                    //in top half of the next to last column
                    if(j==i-1 && k < numRowsOld) {
                        retVal[i][j][k] = 0;
                    }
                    //in bottom half of the next to last column
                    else if(j==i-1 && k >= numRowsOld) {
                        retVal[i][j][k] = 1;
                    }
                    //in some other column -- then copy pattern from
                    //the j+1st truth table
                    else {
                        int pow = powers[j]+powers[j];
                        retVal[i][j][k] = retVal[j+1][j][k % pow];
                    }
                }
            }
        }
        TRUTH_TABLES = retVal;   
    }
    
    public static Object getInstance(String fullName) {
        try {
            Class cl = Class.forName(fullName);
            return cl.newInstance();
        }
        catch(Exception e) {
            return null;
        }
    }
    
}