package sampleCode2;
import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;


public class IteGUI extends JApplet {

    //GUI constants
    private Color mainBgColor = new Color(0x000080);  //dark blue
    private Color topBgColor  = new Color(0xf9c668);  //medium gold
    private Color bottomBgColor = new Color(0xf9e58f); //yellow
    private final String IF_THEN_ELSE = "If-Then-Else Framework";
    private final String NESTED_IF    = "Ordinary Nested-If Code";
    private final String IF_THEN_ELSE_CLASS =  "URLProcessor_good";
    private final String NESTED_IF_CLASS    =  "URLProcessor_bad";
    private final int NUM_FRAMEWORK_CHOICES = 2;
    private final int NUM_REGION_CHOICES = 5;
    private final String EAST = "East";
    private final String WEST = "West";
    private final String SOUTH = "South";
    private final String MIDWEST = "Midwest";
    private final String NORTH   = "North";
    
    //references
    DataBank db;
    URLProcessor up;
    
    //data storage
    Hashtable frameworkChoices = new Hashtable(); 
    Hashtable regionChoices    = new Hashtable();
    String [] codeSelectionModel, userIdModel, regionModel, limitThresholdModel;   
    
    //controls
    JComboBox codeSelection, userId, region, limitThreshold;
    JTextField urlTextField;
    

    public void init() {
        initializeInternalTables();
        createModels();
        initializeComboBoxes();
        createGUI();
    }
    
    private void initializeInternalTables() {
        //initialize framework choices
        frameworkChoices.put(IF_THEN_ELSE,  new URLProcessor_good() );
        frameworkChoices.put(NESTED_IF,     new URLProcessor_bad()  );
        
        //initialize region choices
        regionChoices.put(EAST, DataBank.EAST_REGION);
        regionChoices.put(WEST, DataBank.WEST_REGION);
        regionChoices.put(MIDWEST, DataBank.MIDWEST_REGION);
        regionChoices.put(SOUTH, DataBank.SOUTH_REGION);
        regionChoices.put(NORTH, DataBank.NORTH_REGION);
        
    }    
    
    private void createModels(){
        // codeSelectionModel
        codeSelectionModel = new String[NUM_FRAMEWORK_CHOICES];
        int i = 0;        
        for(Enumeration displayVals = frameworkChoices.keys(); displayVals.hasMoreElements();) {
            codeSelectionModel[i] = (String)displayVals.nextElement();
            ++i;
        }
        
       
        //userIdModel 
        String [] user_id_model =   
                      {"0000",
                       "1500",
                       "3392",
                       "5831",
                       "9091"};
        userIdModel = user_id_model;
                       
        //regionModel
        regionModel = new String[NUM_REGION_CHOICES];
        i = 0;        
        for(Enumeration displayVals = regionChoices.keys(); displayVals.hasMoreElements();) {
            regionModel[i] = (String)displayVals.nextElement();
            ++i;
        }
        regionModel = sort(regionModel);
        
        //limitThresholdModel
        String [] limit_threshold_model = 
                              {"500",
                               "1000",
                               "5000",
                               "10000",
                               "50000"};
        limitThresholdModel = limit_threshold_model;
    }
    
    private void initializeComboBoxes() {
        codeSelection = new JComboBox(codeSelectionModel);
        codeSelection.setSelectedIndex(0);
        
        userId = new JComboBox(userIdModel);
        userId.setSelectedIndex(0);
        
        region = new JComboBox(regionModel);
        region.setSelectedIndex(0);
        
        limitThreshold = new JComboBox(limitThresholdModel);
        limitThreshold.setSelectedIndex(0);
    }
    
    private void createGUI() {
        
        Container contentPane = getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.setBackground(mainBgColor);
        
        JPanel upperPanel = createUpperPanel();
        upperPanel.setBackground(topBgColor);
        
        JPanel lowerPanel = createLowerPanel();
        lowerPanel.setBackground(bottomBgColor);
        
        contentPane.add(upperPanel, BorderLayout.NORTH);
        contentPane.add(lowerPanel, BorderLayout.SOUTH);
        
    }
    
    private JPanel createUpperPanel() {
        JPanel upper= new JPanel();
        upper.setLayout(new FlowLayout(FlowLayout.CENTER,15,15));
        
        //Code Selection
        JLabel codeSelectionLabel = new JLabel("Code Selection");
        upper.add(newLabeledComponent(codeSelectionLabel, codeSelection));
        
        //User ID
        JLabel userIdLabel = new JLabel("User ID");
        upper.add(newLabeledComponent(userIdLabel, userId));
        
        //Region
        JLabel regionLabel = new JLabel("Region");
        upper.add(newLabeledComponent(regionLabel, region));        
        
        //Limit threshold
        JLabel limitThresholdLabel = new JLabel("Limit");
        upper.add(newLabeledComponent(limitThresholdLabel, limitThreshold));
        
        return upper;
        
    }
    
    private JPanel createLowerPanel() {
        JPanel lower = new JPanel();
        lower.setLayout(new BorderLayout());
        
        //URL textfield
        JPanel urlPane = createLabeledUrlTextField();
        urlPane.setOpaque(false);
        lower.add(urlPane, BorderLayout.SOUTH);
        
        //row of buttons
        JPanel buttonPanel = createButtonPanel();
        buttonPanel.setOpaque(false);
        lower.add(buttonPanel, BorderLayout.NORTH);
        
        return lower;
    }
    
    private JPanel createLabeledUrlTextField() {
        JPanel urlPanel = new JPanel();
        
        urlTextField = new JTextField(50);
        urlTextField.setEditable(false);
        urlTextField.setBackground(Color.white);
        Font font = urlTextField.getFont();
        Font newFont = 
            new Font(font.getName(),Font.BOLD,font.getSize()-2);
        urlTextField.setFont(newFont);
        JLabel urlLabel = new JLabel("URL ");
        
        urlPanel.add(urlLabel);
        urlPanel.add(urlTextField);
        
        return urlPanel;
    }
    
    private JPanel createButtonPanel() {
        JPanel butnPanel = new JPanel();
        butnPanel.setLayout(new FlowLayout(FlowLayout.CENTER,8,8));
        
        //find url button
        JButton findUrlButn = new JButton("Find URL");
        findUrlButn.addActionListener(new FindUrlListener());
        
        //connect to url button
        JButton connectButn = new JButton("Connect to URL");
        connectButn.addActionListener(new ConnectListener());
        
        //reset button
        JButton resetButn = new JButton("Reset URL");
        resetButn.addActionListener(new ResetListener());
        
        //add all to panel
        butnPanel.add(findUrlButn);
        butnPanel.add(connectButn);
        butnPanel.add(resetButn);
        
        return butnPanel;
    }
    
    private void resetUrl() {
        urlTextField.setText("");
        repaint();
    }
    
    /**
     *  This listener is the place where the GUI connects
     *  with URLProcessor to invoke the If-Then-Else framework
     */
    class FindUrlListener implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            resetUrl();
            
            //get selected user id
            String id = (String)userId.getSelectedItem();
            
            //get selected limit threshold
            Object ob = limitThreshold.getSelectedItem();
            int limit = 0;
            try {
                limit = (new Integer((String)ob)).intValue();
            }
            catch(NumberFormatException nfe) {
                System.out.println("Number format exception: "+(String)ob);
            }
            
            //get region
            ob = region.getSelectedItem();
            DataBank.Region reg = (DataBank.Region)regionChoices.get(ob);
            
            //set the relevant instance of DataBank
            db = new DataBank(id,limit,reg);
            
            //set the relevant instance of URLProcessor
            String codeSel = (String)codeSelection.getSelectedItem();
            up = (URLProcessor)frameworkChoices.get(codeSel);
            up.setDb(db);
            
            //invoke IfThenElse
            up.decideUrl();
            
            //set value found
            urlTextField.setText(up.getUrl().toString());    
        }
    }
    
    class ConnectListener implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            IteGUI.this.getAppletContext().showDocument(up.getUrl());
        }
    }
    
    class ResetListener implements ActionListener {
        public void actionPerformed(ActionEvent evt) {
            resetUrl();
        }
    }
    
    //----------- UTILITIES -------------//

    /** 
     *  Creates a panel with a label in the top part 
     *  and a JComponent in the bottom part
     */
    public static JPanel newLabeledComponent(JLabel label, JComponent component) {
        //adjust position of label
        String labelText = label.getText();
        labelText = " "+labelText;
        label.setText(labelText);
        
        //create labeled component
        JPanel panel = new JPanel();
        panel.setOpaque(false);
        panel.setLayout(new BoxLayout(panel,BoxLayout.Y_AXIS));
        label.setAlignmentX(Component.LEFT_ALIGNMENT);
        panel.add(label);
        component.setAlignmentX(Component.LEFT_ALIGNMENT);
        panel.add(component);
        return panel;
    }
    
/**
 * Sorts Strings in the array arr and returns it in a new array. The original
 * array remains unmodified.<br>
 * Uses Insertion Sort algorithm -- efficient only for fairly small arrays
 * <p>
 * @author Paul Corazza
 * @param arr String[]
 */

public static String[] sort(String[] arr) {

    //data validation
    if (arr==null || arr.length <= 1) return arr;
    String[] arrCopy = new String[arr.length];
    System.arraycopy(arr,0,arrCopy,0,arr.length);

    //sort
    int n = arrCopy.length;
    String nextUnsorted;
    int indexOfNextUnsorted, index;
    for(indexOfNextUnsorted = 1; indexOfNextUnsorted < n; ++indexOfNextUnsorted){
        nextUnsorted = arrCopy[indexOfNextUnsorted];
        index = indexOfNextUnsorted - 1;

        while(index >= 0 && (nextUnsorted.compareTo(arrCopy[index]) <0)) {
            arrCopy[index+1] = arrCopy[index];
            index--;
        }
        arrCopy[index+1] =nextUnsorted;
    }
    return arrCopy;

}    
    /**
     *  This method is just for use with the main method 
     *  to position the test frame properly on the screen
     */
    private static void positionFrameOnDesktop(JFrame f) {
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        int height = toolkit.getScreenSize().height;
        int width  = toolkit.getScreenSize().width;
        int frameHeight = f.getSize().height;
        int frameWidth  = f.getSize().width;
        f.setLocation((width-frameWidth)/2, (height-frameHeight)/3);        
    }
    
    /**
     *  The main method allows you to run the applet in a JFrame.
     *  However, unless you are running this (as an application)
     *  on a server, you won't be able to connect to the urls,
     *  though you will be able to see the url's in the window
     *  provided.
     *  <p>
     *  For full functionality, run this class as an applet in
     *  a browser by linking to IteGUI.html.
     */
    public static void main(String [] args) {
        
        IteGUI gui = new IteGUI();
        gui.init();
        
        JFrame f = new JFrame("Test JavaWorld Sample");
        f.setSize(400,400);
        f.setBackground(Color.gray);
        f.getContentPane().add(gui);
        f.addWindowListener( new WindowAdapter() {
            public void windowClosing(WindowEvent w) {
                System.exit(0);
            }
        }); 
        f.pack();
        positionFrameOnDesktop(f);
        f.setVisible(true);
        
        
    }

        
} 
    
