package sampleProfile;

import logic.*;
import java.util.*;
import java.io.*;
import java.math.*;

/** Use this class to run performance tests -- see the main method */
public class Main implements Updateable {
    static int numConditions;
    static int numStars;
    static int [] arrayOf0sAnd1s;
    static String newline = System.getProperty("line.separator");
    Algorithm alg;
    Boolean consisCheck;
    Invoker inv;

    Main(int numConditions, int numStars, Algorithm alg, Boolean consisCheck) {
        this.numConditions = numConditions;
        this.numStars = numStars;
        this.alg = alg;
        this.consisCheck = consisCheck;
        createArrayOf0sAnd1s();
    }
    private void createArrayOf0sAnd1s() {
        arrayOf0sAnd1s = new int[numConditions];
        Random r = new Random();
        for(int i = 0; i < numConditions; ++i) {
            arrayOf0sAnd1s[i] = Math.abs(r.nextInt()) % 2;
        }
    }
    
    
    private String appendStars(String s, int numStars) {
        StringBuffer sb = (s == null) ? 
                          new StringBuffer() :
                          new StringBuffer(s);
        for(int i = 0; i < numStars; ++i) {
            sb.append('*');
        }
        return sb.toString();
    }
        
        
    
    public void invoke() {
        try {
            
            inv = new ConcreteInvoker((Updateable)this);
            inv.init();
            inv.execute();
        }
        catch(Exception e) {
            System.out.println(e.getMessage());
        }
    }
    
    public Algorithm getAlgorithmUsed() {
        try {
            return inv.getRules().getAlgorithm();
        }
        catch(Exception e) {
            return null;
        }
    }
    public Boolean getConsisCheckOptionUsed() {
        try {
            return inv.getRules().getConsistencyChecking();
        }
        catch(Exception e ) {
            return null;
        }
    }
    
    public void doUpdate(Hashtable ht) {
        if(ht == null) return;
        Object ob = ht.get(Action.MAIN_KEY);
        if(ob != null && ob instanceof String) {
            System.out.println(newline + (String)ob);
        }
    }
    
    public class ConcreteInvoker extends Invoker {
        public ConcreteInvoker(Updateable ud) throws PoorPerformanceException  {
            super(ud);
        }
        
        public void loadConditions() throws IllegalExpressionException,
                                                RuleNotFoundException {
            conditions = new Vector(numConditions);
            for(int i = 0; i < numConditions; ++i) {
                conditions.addElement(new Condition(arrayOf0sAnd1s[i], 
                                                    Condition.EQUAL, 1));
            }
        }

        public void loadRules() throws InconsistentRulesException, 
                                           PoorPerformanceException, 
                                           RuleNotFoundException  {
                                            
            int maxNumStars = numStars;
            int numNonstars = numConditions-numStars;                                            
            Util.truthTable(numNonstars);
            
            
            
            //we don't want to compute the total number of rules after
            //replacing all stars with Ts and Fs -- we want to count number
            //of rules with stars in them, as user defines them 
            int numRules = (int)Math.pow(2,numNonstars);
            

            rules = new Rules(numConditions,numRules,maxNumStars,alg,consisCheck );
            Action a = new Action(ud, "Action performed.");
            StringBuffer sb = null;
            char next = 'T';
            for(int j = 0; j < numRules; ++j) {
                sb = new StringBuffer(numConditions);
                int i = 0;
                for(i = 0; i < numNonstars; ++i) {
                    
                    next = (Util.TRUTH_TABLES[numNonstars][i][j] == 1) ? 'T' : 'F';
                    sb.append(next);
                }
                for(i = numNonstars; i < numConditions; ++i) {
                    sb.append('*');
                }
                
                rules.addRule(sb.toString(), a);
            }
        }
    }
    
    public static void main(String [] args) {
        
        int numConditions = 12;
        int numStars = 0;
        Boolean consistencyChecking = Boolean.TRUE;
        Algorithm alg = (Algorithm_fewStars)Util.getInstance(Algorithm_fewStars.NAME);
        testPerformance(numConditions,numStars,null,null);
        //testPerformance(numConditions,numStars,alg,consistencyChecking);
        
        //pauses the program so you can read output
        try {
            System.in.read();
        }
        catch(Exception e) {}
    }
    
    
    
    
    /** this tests to see how long it takes to create the
     *  required Boolean tuples
     */
    public static void testPerformance(int numConditions, int numStars, Algorithm alg, Boolean consisCheck) {
        long t1 = Profiler.time();
        Main m = new Main(numConditions, numStars,alg,consisCheck);
        m.invoke();
        long t2 = Profiler.time();
        long duration = Profiler.prof(t1,t2);
        
        String timeToRun = "Time to run = " + (new BigDecimal(duration)).divide(new BigDecimal(1000), 2,BigDecimal.ROUND_HALF_UP)+ " seconds.";
        String algSelected = "User-selected algorithm = " + 
                             (alg == null ? "none selected" : alg.name());
        String algUsed     = "Algorithm used by framework = "+
                              (m.getAlgorithmUsed()==null ? "exception thrown" : m.getAlgorithmUsed().name());
        String consisChk   = "Consistency checking option chosen: "+
                             (consisCheck == null ? "none selected" : consisCheck.toString());
        String consisChkUsed = "Consistency checking option used by framework: "+ 
                                (m.getConsisCheckOptionUsed()==null ? "exception thrown" : m.getConsisCheckOptionUsed().toString());
        String numCondSel    = "Number of conditions selected = " + numConditions;
        String numStarsSel   = "Number of stars selected = "+numStars;
        System.out.println(newline +
                           algSelected+newline + 
                           algUsed + newline +
                           consisChk + newline +
                           consisChkUsed + newline +
                           numCondSel + newline +
                           numStarsSel + newline +
                           timeToRun);
                           
    } 
}

            
            
               
    
    
